@@ -2,25 +2,28 @@
 Model Generator for OpenSees ~ element
 """
 
-#                          __
-#   ____  ____ ___  ____ _/ /
-#  / __ \/ __ `__ \/ __ `/ /
-# / /_/ / / / / / / /_/ /_/
-# \____/_/ /_/ /_/\__, (_)
-#                /____/
+#
+#   _|_|      _|_|_|  _|      _|    _|_|_|
+# _|    _|  _|        _|_|  _|_|  _|
+# _|    _|    _|_|    _|  _|  _|  _|  _|_|
+# _|    _|        _|  _|      _|  _|    _|
+#   _|_|    _|_|_|    _|      _|    _|_|_|
+#
 #
 # https://github.com/ioannis-vm/OpenSees_Model_Generator
 
 from dataclasses import dataclass, field
 from typing import Union
+from typing import Optional
 import numpy as np
 import numpy.typing as npt
 from .uniaxial_material import UniaxialMaterial
 from .node import Node
 from .section import ElasticSection
 from .section import FiberSection
+from ..mesh import Mesh
 from ..graphics.visibility import ElementVisibility
-from .. import component_assembly
+from ..component_assembly import ComponentAssembly
 
 
 nparr = npt.NDArray[np.float64]
@@ -31,10 +34,20 @@ class Element:
     """
     OpenSees element
     https://openseespydoc.readthedocs.io/en/latest/src/element.html
+    Attributes:
+        parent_component (component_assembly.ComponentAssembly):
+          the parent component assembly that this element belongs to
+        uid (int): the unique identifier of this element
+        nodes (list[Node]): the list of nodes that this element connects
     """
-    parent_component: component_assembly.ComponentAssembly = field(repr=False)
+
+    parent_component: ComponentAssembly = field(repr=False)
     uid: int
     nodes: list[Node]
+    visibility: ElementVisibility = field(init=False)
+
+    def __post_init__(self):
+        self.visibility = ElementVisibility()
 
 
 @dataclass(repr=False)
@@ -43,6 +56,7 @@ class ZeroLength(Element):
     OpenSees ZeroLength element
     https://openseespydoc.readthedocs.io/en/latest/src/ZeroLength.html
     """
+
     mats: list[UniaxialMaterial]
     dirs: list[int]
     vecx: nparr
@@ -54,27 +68,27 @@ class ZeroLength(Element):
         OpenSees
         """
         return [
-            'zeroLength',
+            "zeroLength",
             self.uid,
             *[n.uid for n in self.nodes],
-            '-mat',
+            "-mat",
             *[m.uid for m in self.mats],
-            '-dir',
+            "-dir",
             *self.dirs,
-            '-orient',
+            "-orient",
             *self.vecx,
-            *self.vecyp
+            *self.vecyp,
         ]
 
     def __repr__(self):
-        res = ''
-        res += 'ZeroLength element object\n'
-        res += f'uid: {self.uid}'
-        res += 'Materials:'
+        res = ""
+        res += "ZeroLength element object\n"
+        res += f"uid: {self.uid}"
+        res += "Materials:"
         for mat, direction in zip(self.mats, self.dirs):
-            res += f'  {direction}: {mat.name}\n'
-        res += f'vecx: {self.vecx}\n'
-        res += f'vecyp: {self.vecyp}\n'
+            res += f"  {direction}: {mat.name}\n"
+        res += f"vecx: {self.vecx}\n"
+        res += f"vecyp: {self.vecyp}\n"
         return res
 
 
@@ -84,6 +98,7 @@ class TwoNodeLink(Element):
     OpenSees TwoNodeLink element
     https://openseespydoc.readthedocs.io/en/latest/src/twoNodeLink.html
     """
+
     mats: list[UniaxialMaterial]
     dirs: list[int]
     vecx: nparr
@@ -95,27 +110,95 @@ class TwoNodeLink(Element):
         OpenSees
         """
         return [
-            'twoNodeLink',
+            "twoNodeLink",
             self.uid,
             *[n.uid for n in self.nodes],
-            '-mat',
+            "-mat",
             *[m.uid for m in self.mats],
-            '-dir',
+            "-dir",
             *self.dirs,
-            '-orient',
-            *self.vecx,
-            *self.vecyp
+            "-orient",
+            *self.vecyp,
         ]
 
     def __repr__(self):
-        res = ''
-        res += 'TwoNodeLink element object\n'
-        res += f'uid: {self.uid}'
-        res += 'Materials:'
+        res = ""
+        res += "TwoNodeLink element object\n"
+        res += f"uid: {self.uid}"
+        res += "Materials:"
         for mat, direction in zip(self.mats, self.dirs):
-            res += f'  {direction}: {mat.name}\n'
-        res += f'vecx: {self.vecx}\n'
-        res += f'vecyp: {self.vecyp}\n'
+            res += f"  {direction}: {mat.name}\n"
+        res += f"vecx: {self.vecx}\n"
+        res += f"vecyp: {self.vecyp}\n"
+        return res
+
+
+@dataclass(repr=False)
+class TrussBar(Element):
+    """
+    Implements both of the following:
+    OpenSees Truss Element
+    https://openseespydoc.readthedocs.io/en/latest/src/trussEle.html
+    OpenSees Corotational Truss Element
+    https://openseespydoc.readthedocs.io/en/latest/src/corotTruss.html
+
+    """
+
+    transf_type: str
+    area: float
+    mat: UniaxialMaterial
+    outside_shape: Optional[Mesh] = field(default=None)
+    weight_per_length: float = field(default=0.00)
+    rho: float = field(default=0.00)
+    cflag: int = field(default=0)
+    rflag: int = field(default=0)
+
+    def ops_args(self):
+        """
+        Returns the arguments required to define the object in
+        OpenSees
+        """
+        elm_name = {
+            'Linear': 'Truss',
+            'Corotational': 'corotTruss'
+        }
+
+        return [
+            elm_name[self.transf_type],
+            self.uid,
+            self.nodes[0].uid,
+            self.nodes[1].uid,
+            self.area,
+            self.mat.uid,
+            '-rho',
+            self.rho,
+            '-cMass',
+            self.cflag,
+            '-doRayleigh',
+            self.rflag
+        ]
+
+    def clear_length(self):
+        """
+        Returns the clear length of the element (without the rigid
+        offsets)
+        """
+        p_i = np.array(self.nodes[0].coords)
+        p_j = np.array(self.nodes[1].coords)
+        return np.linalg.norm(p_i - p_j)
+
+    def __repr__(self):
+        elm_name = {
+            'Linear': 'Truss',
+            'Corotational': 'corotTruss'
+        }
+        res = ""
+        res += f"{elm_name[self.transf_type]} element object\n"
+        res += f"uid: {self.uid}\n"
+        res += f"node_i.uid: {self.nodes[0].uid}\n"
+        res += f"node_j.uid: {self.nodes[1].uid}\n"
+        res += f"node_i.coords: {self.nodes[0].coords}\n"
+        res += f"node_j.coords: {self.nodes[1].coords}\n"
         return res
 
 
@@ -126,6 +209,7 @@ class GeomTransf:
     https://openseespydoc.readthedocs.io/en/latest/src/ZeroLength.html
     https://openseespydoc.readthedocs.io/en/latest/src/geomTransf.html?highlight=geometric%20transformation
     """
+
     transf_type: str
     uid: int
     offset_i: nparr
@@ -143,9 +227,9 @@ class GeomTransf:
             self.transf_type,
             self.uid,
             *self.z_axis,
-            '-jntOffset',
+            "-jntOffset",
             *self.offset_i,
-            *self.offset_j
+            *self.offset_j,
         ]
 
 
@@ -155,10 +239,9 @@ class ElasticBeamColumn(Element):
     OpenSees Elastic Beam Column Element
     https://openseespydoc.readthedocs.io/en/latest/src/elasticBeamColumn.html
     """
+
     section: ElasticSection
     geomtransf: GeomTransf
-    visibility: ElementVisibility = field(
-        default_factory=ElementVisibility)
 
     def ops_args(self):
         """
@@ -166,7 +249,7 @@ class ElasticBeamColumn(Element):
         OpenSees
         """
         return [
-            'elasticBeamColumn',
+            "elasticBeamColumn",
             self.uid,
             self.nodes[0].uid,
             self.nodes[1].uid,
@@ -176,7 +259,7 @@ class ElasticBeamColumn(Element):
             self.section.j_mod,
             self.section.i_y,
             self.section.i_x,
-            self.geomtransf.uid
+            self.geomtransf.uid,
         ]
 
     def clear_length(self):
@@ -189,19 +272,19 @@ class ElasticBeamColumn(Element):
         return np.linalg.norm(p_i - p_j)
 
     def __repr__(self):
-        res = ''
-        res += 'elasticBeamColumn element object\n'
-        res += f'uid: {self.uid}\n'
-        res += f'node_i.uid: {self.nodes[0].uid}\n'
-        res += f'node_j.uid: {self.nodes[1].uid}\n'
-        res += f'node_i.coords: {self.nodes[0].coords}\n'
-        res += f'node_j.coords: {self.nodes[1].coords}\n'
-        res += f'offset_i: {self.geomtransf.offset_i}\n'
-        res += f'offset_j: {self.geomtransf.offset_j}\n'
-        res += f'x_axis: {self.geomtransf.x_axis}\n'
-        res += f'y_axis: {self.geomtransf.y_axis}\n'
-        res += f'z_axis: {self.geomtransf.z_axis}\n'
-        res += f'section.name: {self.section.name}\n'
+        res = ""
+        res += "elasticBeamColumn element object\n"
+        res += f"uid: {self.uid}\n"
+        res += f"node_i.uid: {self.nodes[0].uid}\n"
+        res += f"node_j.uid: {self.nodes[1].uid}\n"
+        res += f"node_i.coords: {self.nodes[0].coords}\n"
+        res += f"node_j.coords: {self.nodes[1].coords}\n"
+        res += f"offset_i: {self.geomtransf.offset_i}\n"
+        res += f"offset_j: {self.geomtransf.offset_j}\n"
+        res += f"x_axis: {self.geomtransf.x_axis}\n"
+        res += f"y_axis: {self.geomtransf.y_axis}\n"
+        res += f"z_axis: {self.geomtransf.z_axis}\n"
+        res += f"section.name: {self.section.name}\n"
         return res
 
 
@@ -211,6 +294,7 @@ class BeamIntegration:
     OpenSees beamIntegration parent class.
     https://openseespydoc.readthedocs.io/en/latest/src/beamIntegration.html?highlight=beamintegration
     """
+
     uid: int
     parent_section: Union[ElasticSection, FiberSection] = field(repr=False)
 
@@ -221,6 +305,7 @@ class Lobatto(BeamIntegration):
     OpenSees Lobatto beam integration.
     https://openseespydoc.readthedocs.io/en/latest/src/Lobatto.html
     """
+
     n_p: int
 
     def ops_args(self):
@@ -228,12 +313,7 @@ class Lobatto(BeamIntegration):
         Returns the arguments required to define the object in
         OpenSees
         """
-        return [
-            'Lobatto',
-            self.uid,
-            self.parent_section.uid,
-            self.n_p
-        ]
+        return ["Lobatto", self.uid, self.parent_section.uid, self.n_p]
 
 
 @dataclass
@@ -242,11 +322,10 @@ class DispBeamColumn(Element):
     OpenSees dispBeamColumn element
     https://openseespydoc.readthedocs.io/en/latest/src/ForceBeamColumn.html
     """
+
     section: FiberSection
     geomtransf: GeomTransf
     integration: BeamIntegration
-    visibility: ElementVisibility = field(
-        default_factory=ElementVisibility)
 
     def ops_args(self):
         """
@@ -254,13 +333,13 @@ class DispBeamColumn(Element):
         OpenSees
         """
         return [
-            'dispBeamColumn',
+            "dispBeamColumn",
             self.uid,
             self.nodes[0].uid,
             self.nodes[1].uid,
             self.geomtransf.uid,
             self.integration.uid,
-            # '-iter',  # can change it to forceBeamColumn here for testing!
+            # '-iter',  # can change it to forceBeamColumn here for testing
             # 50,
             # 1e-1
         ]
@@ -275,17 +354,17 @@ class DispBeamColumn(Element):
         return np.linalg.norm(p_i - p_j)
 
     def __repr__(self):
-        res = ''
-        res += 'dispBeamColumn element object\n'
-        res += f'uid: {self.uid}\n'
-        res += f'node_i.uid: {self.nodes[0].uid}\n'
-        res += f'node_j.uid: {self.nodes[1].uid}\n'
-        res += f'node_i.coords: {self.nodes[0].coords}\n'
-        res += f'node_j.coords: {self.nodes[1].coords}\n'
-        res += f'offset_i: {self.geomtransf.offset_i}\n'
-        res += f'offset_j: {self.geomtransf.offset_j}\n'
-        res += f'x_axis: {self.geomtransf.x_axis}\n'
-        res += f'y_axis: {self.geomtransf.y_axis}\n'
-        res += f'z_axis: {self.geomtransf.z_axis}\n'
-        res += f'section.name: {self.section.name}\n'
+        res = ""
+        res += "dispBeamColumn element object\n"
+        res += f"uid: {self.uid}\n"
+        res += f"node_i.uid: {self.nodes[0].uid}\n"
+        res += f"node_j.uid: {self.nodes[1].uid}\n"
+        res += f"node_i.coords: {self.nodes[0].coords}\n"
+        res += f"node_j.coords: {self.nodes[1].coords}\n"
+        res += f"offset_i: {self.geomtransf.offset_i}\n"
+        res += f"offset_j: {self.geomtransf.offset_j}\n"
+        res += f"x_axis: {self.geomtransf.x_axis}\n"
+        res += f"y_axis: {self.geomtransf.y_axis}\n"
+        res += f"z_axis: {self.geomtransf.z_axis}\n"
+        res += f"section.name: {self.section.name}\n"
         return res
